  ##########################################################################
##########################################################################
##
## Gizmo Daemon Python Config Script
##
##	gizmoDaemonConfig.py - v2:3
##
#######################################
  #######################################

import GizmoDaemon

import commands
import sys
import string
import time
from array import array
import subprocess

#######################################
# Constants
###################

VERSION = "2:3"
DEVICE_KEYBOARD = "Keyboard"
DEVICE_MOUSE = "Mouse"
DEVICE_ATIX10 = "X10 Wireless Technology"
ALWAYS_CONTROL_VOLUME = False

# enable this to turn on debugging messages
# including print outs of received events
DEBUG = False

# number of installed monitors
MAX_MONITORS = 3

#######################################
# Gizmo Class
###################

class GizmoController:
        #####################
        # contstructor
        #####
        def __init__(self):
                self.processing = True
                self.selectedMonitor = "Volume"
                self.defaultMonitor = "Volume"
                self.lastMonitor = "Volume"
                self.monitorInterrupt = False
                self.foregroundWindow = "Unknown"
                self.soundMakerOpen = False
                self.powermates = 0;
                
                self.Ctrl = False
                self.Alt = False
                self.Meta = False
                self.Shift = False
                
                self.usage()

        #####################
        # initialization
        #####
        def initialize(self):
        	DeviceID = -1
        				
		# load a generic plugin for the keyboard
		if DEBUG: 
			print "    Listing Devices..."
			GizmoDaemon.loadPlugin("GizmoPluginGeneric", "!ListDevices")

		# load XOSD plugin
		print "    Loading XOSD Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginXOSD", "arial;Cyan;30")
		if DeviceID == -1:
			print "        * Failed to Load XOSD Plugin!"

		# load powermate plugin
		print "    Loading Griffin PowerMate Plugin..."
		self.powermates = 0
		while GizmoDaemon.loadPlugin("GizmoPluginPowerMate", "auto") != -1:
			self.powermates += 1
						
		# load alsa mixer plugin
		print "    Loading AlsaMixer Plugin [Master]..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginAlsaMixer", "hw:0;Master")
		if DeviceID == -1:
			print "        * Failed to Load Alsa Mixer Plugin for Mixer [Masetr]!"
		else:
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", 0):
				GizmoDaemon.powerMateSetLEDPercent(0, GizmoDaemon.alsaMixerGetVolume(0))
						
		# load another alsa mixer plugin
		print "    Loading AlsaMixer Plugin [PCM]..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginAlsaMixer", "hw:0;PCM")
		if DeviceID == -1:
			print "        * Failed to Load Alsa Mixer Plugin for Mixer [PCM]!"
		else:
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", 1):
				GizmoDaemon.powerMateSetLEDPercent(1, GizmoDaemon.alsaMixerGetVolume(1))
		
		# load a generic plugin for the keyboard
		print "    Loading Generic Plugin [Keyboard]..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginGeneric", DEVICE_KEYBOARD)
		if DeviceID == -1:
			print "        * Failed to Load Generic Plugin for [Keyboard]!"

		# load a generic plugin for the mouse
		print "    Loading Generic Plugin [Mouse]..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginGeneric", DEVICE_MOUSE)
		if DeviceID == -1:
			print "        * Failed to Load Generic Plugin for [Mouse]!"
			
		# load a generic plugin for the ATI X10 RF Remote
		print "    Loading Generic Plugin [ATI X10 RF Remote]..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginGeneric", DEVICE_ATIX10) 
		if DeviceID == -1:
			print "        * Failed to Load Generic Plugin for [ATI X10 RF Remote]!"
		else:
			GizmoDaemon.genericSetGrabExclusiveAccess(DeviceID, True)
			GizmoDaemon.genericSetKeyRateLimit(DeviceID, 150)

		# load the LIRC plugin
		print "    Loading LIRC Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginLIRC", "auto")
		if DeviceID == -1:
			print "        * Failed to Load LIRC Plugin!"
		else:
			GizmoDaemon.lircSetKeyRateLimit(DeviceID, 2)

		# load the visualization monitor plugin
		print "    Loading Sound Visualization Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginSoundVisualization", "auto")
		if DeviceID == -1:
			print "        * Failed to Load Sound Visualization Plugin!"
			
		# load the CPU Usage plugin
		print "    Loading CPU Usage Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginCPUUsage", "auto")
		if DeviceID == -1:
			print "        * Failed to Load CPU Usage Plugin!"
		else:
	                self.selectedMonitor = "CPUUsage"
	                self.defaultMonitor = "CPUUsage"

		# load the X11 plugin
		print "    Loading X11 Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginX11", ":0.0")
		if DeviceID == -1:
			print "        * Failed to Load X11 Plugin!"
			
		# load the XNNS Control plugin
		print "    Loading XMMS Control Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginXMMSControl", "auto")
		if DeviceID == -1:
			print "        * Failed to Load XMMS Control Plugin!"

		# load the BMP Control plugin
		print "    Loading BMP Control Plugin..."
		DeviceID = GizmoDaemon.loadPlugin("GizmoPluginBMPControl", "auto")
		if DeviceID == -1:
			print "        * Failed to Load BMP Control Plugin!"

		return True

        #####################
        # Event Handlers
        #####

	def onPowerMateRotate(self, DeviceID, Value, ButtonPressed, DevicePath):
		debugPrint("GizmoScript :: OnPowerMateRotate [" + str(DeviceID) + "] [" + DevicePath + "] [Btn = " + str(ButtonPressed) + "] == " + str(Value))

		if DeviceID == 1 or ALWAYS_CONTROL_VOLUME:
			self.setVolume(DeviceID, Value)
			return
		
		if self.Shift and self.Ctrl:
			if GizmoDaemon.isPluginLoaded("GizmoPluginX11", 0):
				if Value > 0:
					GizmoDaemon.x11SetWorkspaceNext(0);
				else:		
					GizmoDaemon.x11SetWorkspacePrev(0);
		if self.Shift:
			if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
				if Value > 0:
					GizmoDaemon.xmmsNextTrack()
				else:
					GizmoDaemon.xmmsPreviousTrack()
			elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
				if Value > 0:
					GizmoDaemon.bmpNextTrack()
				else:
					GizmoDaemon.bmpPreviousTrack()
		else:
			# wheel scrollers
			if ((not string.find(self.foregroundWindow, "Firefox") == -1) or
			   (not string.find(self.foregroundWindow, "Deer Park") == -1) or
			   (not string.find(self.foregroundWindow, "gedit") == -1) or
			   (not string.find(self.foregroundWindow, "nedit") == -1) or
			   (not string.find(self.foregroundWindow, "konqueror") == -1) or
			   (not string.find(self.foregroundWindow, "Mozilla") == -1) or
			   (not string.find(self.foregroundWindow, "KDevelop") == -1)):
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
				if not DeviceID == -1:
					GizmoDaemon.genericSendRelativeEvent(DeviceID, 8, -Value)
			# otherwise change the volume
			else:
				self.setVolume(DeviceID, Value)

        #####################

	def onPowerMatePress(self, DeviceID, DevicePath):
		debugPrint("GizmoScript :: OnPowerMatePress [" + str(DeviceID) + "] [" + DevicePath + "]")
		
        #####################

	def onPowerMateRelease(self, DeviceID, DevicePath):
		debugPrint("GizmoScript :: OnPowerMateRelease [" + str(DeviceID) + "] [" + DevicePath + "]")

        #####################

	def onPowerMateClick(self, DeviceID, DevicePath):
		debugPrint("GizmoScript :: OnPowerMateClick [" + str(DeviceID) + "] [" + DevicePath + "]")

		if self.Ctrl:
			self.selectMonitorNext()
		elif self.Shift:
			if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
				if ( (not GizmoDaemon.xmmsIsPlaying()) and (not GizmoDaemon.xmmsIsPaused()) ):
					GizmoDaemon.xmmsPlay()
				else:
					GizmoDaemon.xmmsStop()
			elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
				if ( (not GizmoDaemon.bmpIsPlaying()) and (not GizmoDaemon.bmpIsPaused()) ):
					GizmoDaemon.bmpPlay()
				else:
					GizmoDaemon.bmpStop()			
		else:
			self.toggleMute(DeviceID)

        #####################

	def onPowerMateLEDChange(self, DeviceID, Value, ButtonPressed, DevicePath):
		debugPrint("GizmoScript :: OnPowerMateLEDChange [" + str(DeviceID) + "] [" + DevicePath + "] [Btn = " + str(ButtonPressed) + "] == " + str(Value))
		
        #####################
        
	def onAlsaMixerEvent(self, DeviceID, VolumePercent, Mute, CardName, MixerName):
		debugPrint("GizmoScript :: OnAlsaMixerEvent [" + str(DeviceID) + "] [" + CardName + "] [" + MixerName + "] [Vol = " + str(VolumePercent) + "] <" + str(Mute) + ">")

		if not self.selectedMonitor == "Volume":
			self.monitorInterrupt = True
			self.selectMonitor("Volume")
			
		if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", DeviceID):
			if not Mute:
				Volume = GizmoDaemon.alsaMixerGetVolume(DeviceID)
				for lp in range(self.powermates):
					if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
						GizmoDaemon.powerMateSetLEDPercent(lp, Volume)
			else:
				for lp in range(self.powermates):
					if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
						GizmoDaemon.powerMateSetLEDPercent(lp, 0)
				
		if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
			if not Mute:
				GizmoDaemon.xosdPrintString(0, "Volume [" + MixerName + "]")
				GizmoDaemon.xosdDrawPercentage(VolumePercent)
			else:
				GizmoDaemon.xosdPrintString(0, "Mute [" + MixerName + "]")
				GizmoDaemon.xosdDrawPercentage(0)

        #####################

	def onGenericPress(self, DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
		if string.find(DeviceName, DEVICE_ATIX10) != -1:
			self.onX10Press(DeviceID, ButtonCode, ButtonString, DevicePath)
			return

		debugPrint("GizmoScript :: onGenericPress [" + str(ButtonCode) + "] [" + DeviceName + "] -- " + ButtonString + " <" + str(DeviceID) + ">")

		if string.find(DeviceName, DEVICE_KEYBOARD) != -1:
			if ButtonString == "KEY_LEFTCTRL" or ButtonString == "KEY_RIGHTCTRL":
				self.Ctrl = True
			elif ButtonString == "KEY_LEFTSHIFT" or ButtonString == "KEY_RIGHTSHIFT":
				self.Shift = True
			elif ButtonString == "KEY_LEFTALT" or ButtonString == "KEY_RIGHTALT":
				self.Alt = True
			elif ButtonString == "KEY_LEFTMETA" or ButtonString == "KEY_RIGHTMETA":
				self.Meta = True
			return

        #####################

	def onGenericRelease(self, DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
		if string.find(DeviceName, DEVICE_ATIX10) != -1:
			self.onX10Release(DeviceID, ButtonCode, ButtonString, DevicePath)
			return

		debugPrint("GizmoScript :: onGenericRelease [" + str(ButtonCode) + "] [" + DeviceName + "] -- " + ButtonString + " <" + str(DeviceID) + ">")
	
		if string.find(DeviceName, DEVICE_KEYBOARD) != -1:
			if ButtonString == "KEY_LEFTCTRL" or ButtonString == "KEY_RIGHTCTRL":
				self.Ctrl = False
			elif ButtonString == "KEY_LEFTSHIFT" or ButtonString == "KEY_RIGHTSHIFT":
				self.Shift = False
			elif ButtonString == "KEY_LEFTALT" or ButtonString == "KEY_RIGHTALT":
				self.Alt = False
			elif ButtonString == "KEY_LEFTMETA" or ButtonString == "KEY_RIGHTMETA":
				self.Meta = False
			return

        #####################

	def onGenericClick(self, DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
		debugPrint("GizmoScript :: onGenericClick [" + str(ButtonCode) + "] [" + DeviceName + "] -- " + ButtonString + " <" + str(DeviceID) + ">")
			
		if string.find(DeviceName, DEVICE_ATIX10) != -1:
			self.onX10Click(DeviceID, ButtonCode, ButtonString, DevicePath)
			return
		elif string.find(DeviceName, DEVICE_KEYBOARD) != -1:
			# do keyboard special keys
			if ((ButtonString == "KEY_F12" and self.Ctrl) or (ButtonString == "KEY_VOLUMEUP")):
				self.setVolume(0, 1)
			elif ((ButtonString == "KEY_F11" and self.Ctrl) or (ButtonString == "KEY_VOLUMEDOWN")):
				self.setVolume(0, -1)
			elif ((ButtonString == "KEY_F10" and self.Ctrl) or (ButtonString == "KEY_VOLUMEMUTE")):
				self.toggleMute(0)
			elif ((ButtonString == "KEY_F9" and self.Ctrl) or (ButtonString == "KEY_WWW")):
				pid = subprocess.Popen(["firefox", "http://gizmod.sourceforge.net"]).pid
			elif ButtonString == "KEY_MAIL":
				pid = subprocess.Popen(["thunderbird"]).pid
			elif ((ButtonString == "KEY_STOP") or (ButtonString == "KEY_STOPCD")):
				if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
					GizmoDaemon.xmmsStop()
				elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
					GizmoDaemon.bmpStop()			
			elif ButtonString == "KEY_UNDO":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Z", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Z", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_COPY":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_OPEN":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_PASTE":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_FIND":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_CUT":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_X", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_X", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTCTRL", 0)
				return
			elif ButtonString == "KEY_BACK":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_BACKSPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_BACKSPACE", 0)
				return
			elif ButtonString == "KEY_FORWARD":
				# -- not sure what key makes that happen
				pass
			elif ButtonString == "KEY_NEXTSONG":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
					GizmoDaemon.xmmsNextTrack()
				elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
					GizmoDaemon.bmpNextTrack()
			elif ButtonString == "KEY_PLAYPAUSE":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
					if ( (not GizmoDaemon.xmmsIsPlaying()) and (not GizmoDaemon.xmmsIsPaused()) ):
						GizmoDaemon.xmmsPlay()
					else:
						GizmoDaemon.xmmsStop()
				elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
					if ( (not GizmoDaemon.bmpIsPlaying()) and (not GizmoDaemon.bmpIsPaused()) ):
						GizmoDaemon.bmpPlay()
					else:
						GizmoDaemon.bmpStop()			
			elif ButtonString == "KEY_MUTE":
				self.toggleMute(0)
			elif ButtonString == "KEY_PREVIOUSSONG":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
					GizmoDaemon.xmmsPrevTrack()
				elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
					GizmoDaemon.bmpPrevTrack()
			elif ((ButtonString == "KEY_PLAY") or (ButtonString == "KEY_PLAYCD")):
				if GizmoDaemon.isPluginLoaded("GizmoPluginXMMSControl", 0):
					GizmoDaemon.xmmsPlay()
				elif GizmoDaemon.isPluginLoaded("GizmoPluginBMPControl", 0):
					GizmoDaemon.bmpPlay()
			return
			
        #####################

	def onGenericRelative(self, DeviceID, Axis, Value, DeviceName, DevicePath):		
		if string.find(DeviceName, DEVICE_ATIX10) != -1:
			self.onX10Relative(DeviceID, Axis, Value, DevicePath)
			return
			
		debugPrint("GizmoScript :: onGenericRelative [" + str(Axis) + "] [" + str(Value) + "] <" + DeviceName + " -- " + DevicePath + "> <" + str(DeviceID) + ">")

        #####################

	def onGenericAbsolute(self, DeviceID, Axis, Value, DeviceName, DevicePath):
		debugPrint("GizmoScript :: onGenericAbsolute [" + str(Axis) + "] [" + str(Value) + "] <" + DeviceName + " -- " + DevicePath + "> <" + str(DeviceID) + ">")
			
        #####################

	def onGenericMisc(self, DeviceID, Axis, Value, DeviceName, DevicePath):
		debugPrint("GizmoScript :: onGenericMisc [" + str(Axis) + "] [" + str(Value) + "] <" + DeviceName + " -- " + DevicePath + "> <" + str(DeviceID) + ">")
			
        #####################

	def onX10Press(self, DeviceID, ButtonCode, ButtonString, DevicePath):
		debugPrint("GizmoScript :: onX10Press [" + str(ButtonCode) + "] -- " + ButtonString + " <" + str(DeviceID) + ">")

		if ButtonCode == 272:
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
			if not DeviceID == -1:
				GizmoDaemon.genericSendKeyEvent(DeviceID, 272, 1)
		elif ButtonCode == 273:
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
			if not DeviceID == -1:
				GizmoDaemon.genericSendKeyEvent(DeviceID, 273, 1)

        #####################

	def onX10Release(self, DeviceID, ButtonCode, ButtonString, DevicePath):
		debugPrint("GizmoScript :: onX10Release [" + str(ButtonCode) + "] -- " + ButtonString + " <" + str(DeviceID) + ">")

		if ButtonCode == 272:
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
			if not DeviceID == -1:
				GizmoDaemon.genericSendKeyEvent(DeviceID, 272, 0)
		elif ButtonCode == 273:
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
			if not DeviceID == -1:
				GizmoDaemon.genericSendKeyEvent(DeviceID, 273, 0)	

        #####################

	def onX10Click(self, DeviceID, ButtonCode, ButtonString, DevicePath):
		debugPrint("GizmoScript :: onX10Click [" + str(ButtonCode) + "] -- " + ButtonString + " <" + str(DeviceID) + ">")
		
		# remote button map for MPlayer
		if string.find(self.foregroundWindow, "MPlayer") != -1:
			if ButtonString == "KEY_PAUSE":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_PLAY":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_STOP":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_POWER":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 0)
				return
			elif ButtonString == "KEY_BOOKMARKS":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 0)
				return
			elif ButtonString == "KEY_EDIT":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 0)
				return
			elif ButtonString == "KEY_REWIND":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 0)
				return
			elif ButtonString == "KEY_FORWARD":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 0)
				return
			elif ButtonString == "KEY_WWW":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
					Time = time.strftime("%H:%M:%S",time.localtime())
					GizmoDaemon.xosdPrintString(0, "Time: " + Time)
					GizmoDaemon.xosdPrintString(1, "")
				return
			elif ButtonString == "KEY_COFFEE":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Q", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Q", 0)
				return
			elif ButtonString == "KEY_C":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_MINUS", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_MINUS", 0)
				return
			elif ButtonString == "KEY_D":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_EQUAL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_EQUAL", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 0)
				return

		# remote button map for xine
		if string.find(self.foregroundWindow, "xine") != -1:
			if ButtonString == "KEY_PAUSE":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_PLAY":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_STOP":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "KEY_POWER":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 0)
				return
			elif ButtonString == "KEY_BOOKMARKS":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 0)
				return
			elif ButtonString == "KEY_EDIT":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 0)
				return
			elif ButtonString == "KEY_REWIND":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 0)
				return
			elif ButtonString == "KEY_FORWARD":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 0)
				return
			elif ButtonString == "KEY_WWW":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
					Time = time.strftime("%H:%M:%S",time.localtime())
					GizmoDaemon.xosdPrintString(0, "Time: " + Time)
					GizmoDaemon.xosdPrintString(1, "")
				return
			elif ButtonString == "KEY_COFFEE":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Q", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_Q", 0)
				return
			elif ButtonString == "KEY_C":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_MINUS", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_MINUS", 0)
				return
			elif ButtonString == "KEY_D":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_EQUAL", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_EQUAL", 0)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 0)
				return

		# global remote button map
		if ButtonString == "KEY_A":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_A", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_A", 0)
		elif ButtonString == "KEY_B":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_B", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_B", 0)
		elif ButtonString == "KEY_POWER":
			pid = subprocess.Popen(["killall", "xscreensaver"]).pid
			pid = subprocess.Popen(["sudo", "mythfrontend"]).pid
		elif ButtonString == "KEY_PROG1":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 0)
		elif ButtonString == "KEY_PROG2":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 0)
		elif ButtonString == "KEY_WWW":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 0)
		elif ButtonString == "KEY_BOOKMARKS":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_N", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_N", 0)
		elif ButtonString == "KEY_EDIT":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_I", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_I", 0)
		elif ButtonString == "KEY_VOLUMEUP":
			self.setVolume(0, 1)
			self.setVolume(0, 1)
		elif ButtonString == "KEY_VOLUMEDOWN":
			self.setVolume(0, -1)
			self.setVolume(0, -1)
		elif ButtonString == "KEY_MUTE":
			self.toggleMute(0)
		elif ButtonCode == 402: # channel down
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 0)
		elif ButtonCode == 403: # channel up
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 0)
		elif ButtonString == "KEY_1":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_1", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_1", 0)
		elif ButtonString == "KEY_2":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_2", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_2", 0)
		elif ButtonString == "KEY_3":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_3", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_3", 0)
		elif ButtonString == "KEY_4":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_4", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_4", 0)
		elif ButtonString == "KEY_5":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_5", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_5", 0)
		elif ButtonString == "KEY_6":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_6", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_6", 0)
		elif ButtonString == "KEY_7":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_7", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_7", 0)
		elif ButtonString == "KEY_8":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_8", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_8", 0)
		elif ButtonString == "KEY_9":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_9", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_9", 0)
		elif ButtonString == "KEY_0":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_0", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_0", 0)
		elif ButtonString == "KEY_MENU":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 0)
		elif ButtonString == "KEY_KPENTER":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_END", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_END", 0)
		elif ButtonString == "KEY_C":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 0)
		elif ButtonString == "KEY_D":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_D", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_D", 0)
		elif ButtonString == "KEY_COFFEE":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 0)
		elif ButtonString == "KEY_LEFT":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFT", 0)
		elif ButtonString == "KEY_RIGHT":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHT", 0)
		elif ButtonString == "KEY_UP":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 0)
		elif ButtonString == "KEY_DOWN":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 0)
		elif ButtonString == "KEY_ENTER" or ButtonCode == 352:
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ENTER", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ENTER", 0)
		elif ButtonString == "KEY_FRONT":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_HOME", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_HOME", 0)
		elif ButtonString == "KEY_E":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_H", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_H", 0)
		elif ButtonString == "KEY_F":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 0)
		elif ButtonString == "KEY_REWIND":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_COMMA", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_COMMA", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 0)
		elif ButtonString == "KEY_PLAYCD":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 0)
		elif ButtonString == "KEY_FORWARD":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOT", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 0)
		elif ButtonString == "KEY_RECORD":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RECORD", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RECORD", 0)
		elif ButtonString == "KEY_STOP":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_STOP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_STOP", 0)
		elif ButtonString == "KEY_PAUSE":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 0)
									
        #####################

	def onX10Relative(self, DeviceID, Axis, Value, DevicePath):
		debugPrint("GizmoScript :: onX10Relative [" + str(Axis) + "] [" + str(Value) + "] <" + DevicePath + "> <" + str(DeviceID) + ">")
		
		DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_MOUSE)
		if not DeviceID == -1:
			GizmoDaemon.genericSendRelativeEvent(DeviceID, Axis, Value)

        #####################

	def onLIRCClick(self, DeviceID, ButtonCode, ButtonValue, ButtonString, DeviceName, DevicePath):
		debugPrint("GizmoScript :: onLIRCClick [" + ButtonString + "] [" + DeviceName + "] <" + str(ButtonValue) + ">")

		# remote button map for MPlayer
		if string.find(self.foregroundWindow, "MPlayer") != -1:
			if ButtonString == "Pause":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "Play":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SPACE", 0)
				return
			elif ButtonString == "Power":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 0)
				return
			elif ButtonString == "Videos":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_V", 0)
				return
			elif ButtonString == "Menu/i":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_O", 0)
				return
			elif ButtonString == "Replay/SkipRewind":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 0)
				return
			elif ButtonString == "SkipForward":
				DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
				if DeviceID != -1:
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 1)
					GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 0)
				return
			elif ButtonString == "Red":
				if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
					Time = time.strftime("%H:%M:%S",time.localtime())
					GizmoDaemon.xosdPrintString(0, "Time: " + Time)
					GizmoDaemon.xosdPrintString(1, "")
				return
			
		# global remote button map
		if ButtonString == "Go":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_A", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_A", 0)
		elif ButtonString == "Power":
			pid = subprocess.Popen(["killall", "xscreensaver"]).pid
			pid = subprocess.Popen(["sudo", "mythfrontend"]).pid
		elif ButtonString == "TV":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_B", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_B", 0)
		elif ButtonString == "Videos":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 0)
		elif ButtonString == "Music":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_SLASH", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHTSHIFT", 0)
		elif ButtonString == "Pictures":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 0)
		elif ButtonString == "Radio":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_N", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_N", 0)
		elif ButtonString == "Guide":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 0)
		elif ButtonString == "Vol+":
			self.setVolume(0, 1)
			self.setVolume(0, 1)
		elif ButtonString == "Vol-":
			self.setVolume(0, -1)
			self.setVolume(0, -1)
		elif ButtonString == "Mute":
			self.toggleMute(0)
		elif ButtonString == "Ch-":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 0)
		elif ButtonString == "Ch+":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 0)
		elif ButtonString == "1":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_1", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_1", 0)
		elif ButtonString == "2":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_2", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_2", 0)
		elif ButtonString == "3":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_3", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_3", 0)
		elif ButtonString == "4":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_4", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_4", 0)
		elif ButtonString == "5":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_5", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_5", 0)
		elif ButtonString == "6":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_6", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_6", 0)
		elif ButtonString == "7":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_7", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_7", 0)
		elif ButtonString == "8":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_8", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_8", 0)
		elif ButtonString == "9":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_9", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_9", 0)
		elif ButtonString == "0":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_0", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_0", 0)
		elif ButtonString == "#":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_W", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_W", 0)
		elif ButtonString == "*":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_F", 0)
		elif ButtonString == "Menu/i":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_M", 0)
		elif ButtonString == "Back/Exit":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ESC", 0)
		elif ButtonString == "Left":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFT", 0)
		elif ButtonString == "Right":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RIGHT", 0)
		elif ButtonString == "Up":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_UP", 0)
		elif ButtonString == "Down":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOWN", 0)
		elif ButtonString == "OK":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ENTER", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_ENTER", 0)
		elif ButtonString == "Prev.Ch":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_H", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_H", 0)
		elif ButtonString == "Rewind":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_COMMA", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_COMMA", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 0)
		elif ButtonString == "Play":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 0)
		elif ButtonString == "Forward":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOT", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_DOT", 0)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_LEFTSHIFT", 0)
		elif ButtonString == "Replay/SkipBackward":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEDOWN", 0)
		elif ButtonString == "SkipForward":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_PAGEUP", 0)
		elif ButtonString == "Record":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RECORD", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_RECORD", 0)
		elif ButtonString == "Stop":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_STOP", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_STOP", 0)
		elif ButtonString == "Pause":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_P", 0)
		elif ButtonString == "Red":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_END", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_END", 0)
		elif ButtonString == "Green":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_HOME", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_HOME", 0)
		elif ButtonString == "Yellow":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_C", 0)
		elif ButtonString == "Blue":
			DeviceID = GizmoDaemon.isPluginLoadedString("GizmoPluginGeneric", DEVICE_KEYBOARD)
			if DeviceID != -1:
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_I", 1)
				GizmoDaemon.genericSendKeyEventString(DeviceID, "KEY_I", 0)

        #####################

	def onSoundVisualization(self, FrameID, LeftChannel, RightChannel, Combined, Special):
		#debugPrint("FID = " + str(FrameID) + " L = " + str(LeftChannel) + " S = " + str(Special))
		if Special == 1:
			self.soundMakerOpen = True
			self.selectMonitor("SoundVisualization")
			return
		if Special == 2:
			self.soundMakerOpen = False
			self.selectMonitor(self.defaultMonitor)
			return
		if Special == 0:
			if self.selectedMonitor != "SoundVisualization":
				return
				
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", 0):
				GizmoDaemon.powerMateSetLED(0, LeftChannel)
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", 1):
				GizmoDaemon.powerMateSetLED(1, RightChannel)
		else:
			print "GizmoScript :: Sound Maker Unknown!"
			return

        #####################

	def onCPUUsage(self, Usage, UsagePercent):
		if self.selectedMonitor != "CPUUsage":
			return
				
		for lp in range(self.powermates):
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
				GizmoDaemon.powerMateSetLED(lp, Usage)

        #####################

	def onX11ForegroundWindowChange(self, DeviceID, X11Display, WindowName):
		debugPrint("GizmoScript :: onX11ForegroundWindowChange [" + WindowName + "]")
		self.foregroundWindow = WindowName

        #####################

	def onPulse(self, Pulse):
		if self.selectedMonitor != "Pulse":
			return
				
		for lp in range(self.powermates):
			if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
				GizmoDaemon.powerMateSetLED(lp, Pulse)

        #####################
        # main loop
        #####
        
        def enterLoop(self):
		# initialize
		if not self.initialize():
			print "** Failed to Initialize!"
			return
		
		# process events
                print "\nProcessing Events..."
                while self.processing:
                        if GizmoDaemon.waitForNextEvent():
				# some event is occuring!
				debugPrint("GizmoDaemon Config Script :: Shutting Down...")
				self.processing = False;
			else:
				# do user processing
				if self.monitorInterrupt:
					self.selectMonitor("last")
									
        #####################
        # usage
        #####

        def usage(self):
                print "        - GizmoDaemon Config Script -- v2:0"

        #####################
        # set volume 
        #####

	def setVolume(self, DeviceID, Value):
		if not GizmoDaemon.isPluginLoaded("GizmoPluginAlsaMixer", DeviceID):
			return
		if GizmoDaemon.alsaMixerGetMute(DeviceID):
			return
		if not self.selectedMonitor == "Volume":
			self.monitorInterrupt = True
			self.selectMonitor("Volume")
		Level = GizmoDaemon.alsaMixerGetVolume(DeviceID) + Value
		if Level >= 100 or Level <= 0:
			if Level <= 0:
				Level = 0
			else:
				Level = 100
			for lp in range(self.powermates):
				if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
					GizmoDaemon.powerMateSetLED(lp, Level)
					GizmoDaemon.powerMateBlinkLED(lp)
		else:
			for lp in range(self.powermates):
				if GizmoDaemon.isPluginLoaded("GizmoPluginPowerMate", lp):
					GizmoDaemon.powerMateSetLED(lp, Level)
		GizmoDaemon.alsaMixerSetVolume(DeviceID, Level)

		if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
			GizmoDaemon.xosdPrintString(0, "Volume [" + GizmoDaemon.alsaMixerGetMixerName(DeviceID) + "]")
			GizmoDaemon.xosdDrawPercentage(Level)
			
        #####################
        # toggle mute
        #####

	def toggleMute(self, DeviceID):
		if not self.selectedMonitor == "Volume":
			self.monitorInterrupt = True
			self.selectMonitor("Volume")
		if GizmoDaemon.isPluginLoaded("GizmoPluginAlsaMixer", DeviceID):
			Muted = GizmoDaemon.alsaMixerGetMute(DeviceID)
			if Muted:
				Muted = False;
			else:
				Muted = True;
			GizmoDaemon.alsaMixerSetMute(DeviceID, Muted)
			
        #####################
        # selectMonitor
        #####

        def selectMonitor(self, Monitor):
        	if Monitor == "last":
        		self.selectMonitor(self.lastMonitor)
	        	if self.monitorInterrupt:
	        		self.monitorInterrupt = False
        		return
        	self.lastMonitor = self.selectedMonitor
        	self.selectedMonitor = Monitor
        	if self.monitorInterrupt:
        		return
		if GizmoDaemon.isPluginLoaded("GizmoPluginXOSD", 0):
			GizmoDaemon.xosdPrintString(0, "Selected Monitor: " + self.selectedMonitor)      	
        
        #####################
        # getMonitorIntFromString
        #####

        def getMonitorIntFromString(self):
        	if self.selectedMonitor == "Volume":
        		return 0
        	if self.selectedMonitor == "SoundVisualization":
        		return 1
        	if self.selectedMonitor == "CPUUsage":
        		return 2
        	if self.selectedMonitor == "Pulse":
        		return 3
        	return -1

        #####################
        # selectMonitorNext
        #####

        def selectMonitorNext(self):
        	CurMon = self.getMonitorIntFromString()
        	CurMon += 1
        	if (CurMon > MAX_MONITORS):
        		CurMon = 0
        	if CurMon == 0:
        		if not GizmoDaemon.isPluginLoaded("GizmoPluginAlsaMixer", 0):
        			CurMon += 1
        		else:
        			self.selectMonitor("Volume")
        	if CurMon == 1:
        		if not self.soundMakerOpen:
        			CurMon += 1
        		elif not GizmoDaemon.isPluginLoaded("GizmoPluginSoundVisualization", 0):
        			CurMon += 1
        		else:
        			self.selectMonitor("SoundVisualization")
        	if CurMon == 2:
        		if not GizmoDaemon.isPluginLoaded("GizmoPluginCPUUsage", 0):
        			CurMon += 1
        		else:
        			self.selectMonitor("CPUUsage")
        	if CurMon == 3:
        		if not GizmoDaemon.isPluginLoaded("GizmoPluginPulse", 0):
        			CurMon += 1
        		else:
        			self.selectMonitor("Pulse")
			
#######################################
# Class Instance
###################

GizmoScript = GizmoController()

#######################################
# Event Handlers
###################

def onPowerMateRotate(DeviceID, Value, ButtonPressed, DevicePath):
	GizmoScript.onPowerMateRotate(DeviceID, Value, ButtonPressed, DevicePath)

def onPowerMatePress(DeviceID, DevicePath):
	GizmoScript.onPowerMatePress(DeviceID, DevicePath)

def onPowerMateRelease(DeviceID, DevicePath):
	GizmoScript.onPowerMateRelease(DeviceID, DevicePath)

def onPowerMateClick(DeviceID, DevicePath):
	GizmoScript.onPowerMateClick(DeviceID, DevicePath)

def onPowerMateLEDChange(DeviceID, Value, ButtonPressed, DevicePath):
	GizmoScript.onPowerMateLEDChange(DeviceID, Value, ButtonPressed, DevicePath)
	
def onAlsaMixerEvent(DeviceID, VolumePercent, Mute, CardName, MixerName):
	GizmoScript.onAlsaMixerEvent(DeviceID, VolumePercent, Mute, CardName, MixerName)
	
def onGenericPress(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
	GizmoScript.onGenericPress(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath)

def onGenericRelease(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
	GizmoScript.onGenericRelease(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath)
	
def onGenericClick(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath):
	GizmoScript.onGenericClick(DeviceID, ButtonCode, ButtonString, DeviceName, DevicePath)
	
def onGenericRelative(DeviceID, Axis, Value, DeviceName, DevicePath):
	GizmoScript.onGenericRelative(DeviceID, Axis, Value, DeviceName, DevicePath)

def onGenericAbsolute(DeviceID, Axis, Value, DeviceName, DevicePath):
	GizmoScript.onGenericAbsolute(DeviceID, Axis, Value, DeviceName, DevicePath)
	
def onGenericMisc(DeviceID, Axis, Value, DeviceName, DevicePath):
	GizmoScript.onGenericMisc(DeviceID, Axis, Value, DeviceName, DevicePath)

def onSoundVisualization(FrameID, LeftChannel, RightChannel, Combined, Special):
	GizmoScript.onSoundVisualization(FrameID, LeftChannel, RightChannel, Combined, Special)
	
def onCPUUsage(Usage, UsagePercent):
	GizmoScript.onCPUUsage(Usage, UsagePercent)
	
def onX11ForegroundWindowChange(DeviceID, X11Display, WindowName):
	GizmoScript.onX11ForegroundWindowChange(DeviceID, X11Display, WindowName)

def onLIRCClick(DeviceID, ButtonCode, ButtonValue, ButtonString, DeviceName, DevicePath):
	GizmoScript.onLIRCClick(DeviceID, ButtonCode, ButtonValue, ButtonString, DeviceName, DevicePath)

def onPulse(Pulse):
	GizmoScript.onPulse(Pulse)

#######################################
# Debug
###################

def debugPrint(text):
        if DEBUG:
                print text

#######################################
# Main
###################

def gizmoMain():
	if not GizmoDaemon.getVersion() == VERSION:
		print "        * Gizmo Daemon Has Incorrect Version: v" + GizmoDaemon.getVersion() + " Script is: v" + VERSION + "\n"
	else:
	        GizmoScript.enterLoop()
