  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionIsPluginLoaded.cpp
//
//	ScriptFunctionIsPluginLoaded class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionIsPluginLoaded.hpp"
#include "../libGizmo/Debug.hpp"
#include "../libGizmo/UtilTime.hpp"
#include "../libGizmo/PluginLoader.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionIsPluginLoaded::executeFunction() {
	//Debug::outDebug("ScriptFunctionIsPluginLoaded :: executeFunction");
	
	ScriptFuncArg * pRetArg = getOutArg(0);
	if (!pRetArg) {
		Debug::out("ScriptFunctionIsPluginLoaded :: executeFunction -- No Out Parameter!");
		return false;
	}
	pRetArg->ArgBoolean = false;
	
	ScriptFuncArg * pInArg1 = getInArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionIsPluginLoaded :: executeFunction -- No Parameter 0!");
		return false;
	}
	
	ScriptFuncArg * pInArg2 = getInArg(1);
	if (!pInArg2) {
		Debug::out("ScriptFunctionIsPluginLoaded :: executeFunction -- No Parameter 0!");
		return false;
	}
	
	// find out if plugin and device id are lodaded
	if (m_pPluginList->getSize() == 0)
		return true;
	m_pPluginList->moveToHead();
	do {
		GizmoPlugin * pPlugin = m_pPluginList->getData();
		if (pPlugin->getPluginName() != pInArg1->ArgString)
			continue;
		if (pPlugin->getDeviceID() != pInArg2->ArgNumber)
			continue;
			
		// found!
		pRetArg->ArgBoolean = true;
		return true;
	} while (m_pPluginList->moveNext());
	
	// not found
	pRetArg->ArgBoolean = false;	
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionIsPluginLoaded::ScriptFunctionIsPluginLoaded(LinkList<GizmoPlugin> * pPluginList) {
	FunctionName = "isPluginLoaded";
	FunctionDescription = "Checks if a Gizmo Daemon Plugin is Loaded";
	
	m_pScriptEngine = NULL;
	m_pPluginList = pPluginList;
	
	addInArg(SCRIPT_ARG_STRING);		// plugin name
	addInArg(SCRIPT_ARG_NUMBER);		// device id
	addOutArg(SCRIPT_ARG_BOOLEAN, false);	// is it loaded
}

ScriptFunctionIsPluginLoaded::~ScriptFunctionIsPluginLoaded() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void ScriptFunctionIsPluginLoaded::setScriptEngine(ScriptEngineInterface * pScriptEngine) {
	m_pScriptEngine = pScriptEngine;
}
