  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionLoadPlugin.cpp
//
//	ScriptFunctionLoadPlugin class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionLoadPlugin.hpp"
#include "../libGizmo/Debug.hpp"
#include "../libGizmo/UtilTime.hpp"
#include "../libGizmo/PluginLoader.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionLoadPlugin::executeFunction() {
	Debug::outDebug("ScriptFunctionLoadPlugin :: executeFunction");
	
	ScriptFuncArg * pRetArg = getOutArg(0);
	if (!pRetArg) {
		Debug::out("ScriptFunctionLoadPlugin :: executeFunction -- No Out Parameter!");
		return false;
	}
	pRetArg->ArgNumber = -1;
	
	ScriptFuncArg * pArgPluginName = getInArg(0);
	if (!pArgPluginName) {
		Debug::out("ScriptFunctionLoadPlugin :: executeFunction -- No Parameter 0!");
		return false;
	}
	
	ScriptFuncArg * pArgPluginArg1 = getInArg(1);
	if (!pArgPluginArg1) {
		Debug::out("ScriptFunctionLoadPlugin :: executeFunction -- No Parameter 1!");
		return false;
	}
		
	// set the arguments
	LinkList<void> ArgList;
	ArgList.addTail((void *) m_pScriptEngine);
	ArgList.addTail((void *) &pArgPluginArg1->ArgString);
	
	// load the plugin
	GizmoPlugin * pPlugin;
	PluginLoader<GizmoPlugin> * pPluginLoader = new PluginLoader<GizmoPlugin>;
	if ((pPlugin = pPluginLoader->loadPlugin(pArgPluginName->ArgString, &ArgList)) == NULL) {
		Debug::outDebug("ScriptFunctionLoadPlugin :: Failed to Load Plugin [%s]", pArgPluginName->ArgString.getString());
		return true;
	}
	ArgList.deleteAll();
	
	m_pPluginList->addTail(pPlugin);
	m_PluginLoaderList.addTail(pPluginLoader);
	
	pRetArg->ArgNumber = pPlugin->getDeviceID();	
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionLoadPlugin::ScriptFunctionLoadPlugin(LinkList<GizmoPlugin> * pPluginList) {
	FunctionName = "loadPlugin";
	FunctionDescription = "Loads a Gizmo Daemon Plugin";
	
	m_pScriptEngine = NULL;
	m_pPluginList = pPluginList;
	
	addInArg(SCRIPT_ARG_STRING);
	addInArg(SCRIPT_ARG_STRING);
	addOutArg(SCRIPT_ARG_NUMBER, false);
}

ScriptFunctionLoadPlugin::~ScriptFunctionLoadPlugin() {
	unloadPlugins();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void ScriptFunctionLoadPlugin::setScriptEngine(ScriptEngineInterface * pScriptEngine) {
	m_pScriptEngine = pScriptEngine;
}

void ScriptFunctionLoadPlugin::unloadPlugins() {
	Debug::outDebug("ScriptFunctionLoadPlugin :: Unloading Plugin Loaders...");
	if (m_PluginLoaderList.getSize() > 0) {
		m_PluginLoaderList.moveToHead();
		do {
			PluginLoader<GizmoPlugin> * pPluginLoader = (PluginLoader<GizmoPlugin> *) m_PluginLoaderList.getData();
			pPluginLoader->closePlugin();
			delete pPluginLoader;
		} while (m_PluginLoaderList.moveNext());
		m_PluginLoaderList.deleteAll();
	}
}

bool ScriptFunctionLoadPlugin::checkPluginsForPendingEvents() {
	if (m_pPluginList->getSize() == 0)
		return false;
	
	bool ret = false;
	for (int lp = 0; lp < m_pPluginList->getSize(); lp ++) {
		if (!m_pPluginList->moveTo(lp))
			continue;
			
		GizmoPlugin * pPlugin = m_pPluginList->getData();
		if (pPlugin->checkEventsWaiting())
			ret = true;		
	}
	
	return ret;
}
