////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// CommandLineOptions.hpp
//
//	CommandLineOptions header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __CommandLineOptions_h
#define __CommandLineOptions_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "LinkList.hpp"
#include "String.hpp"

//////////////////////////////////////////////////////////////////////////////
// Type Defs
///////////////////////////////////////

enum CLOptionCode {
	CLOPTION_STANDALONE = -1,
	CLOPTION_ERROR = -2
};

struct CLOptionNode {
	String	arg;
};

struct CLOptionRule {
	char	shortOption;
	String	longOption;
	bool	hasArgument;
};

struct CLOptionArg {
	int	shortOption;
	String	argument;
	bool	error;
};

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class CommandLineOptions {
public:
	void	addOptionRule(char shortOption, String LongOption, bool hasArgument);
	bool	createOptionList(int argc, char * argv []);
	void	deleteArgList();
	void	deleteOptionList();
	void	deleteRuleList();
	int	getNumOptions();
	CLOptionArg * getOption(int index);
	bool 	processCommandLine();
	
	CommandLineOptions(int argc, char * argv []);
	virtual ~CommandLineOptions();

private:
	LinkList<CLOptionNode>	m_OptionList;
	LinkList<CLOptionRule>	m_RuleList;
	LinkList<CLOptionArg>	m_ArgList;
};

#endif // __CommandLineOptions_h

