  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// Debug.hpp
//
//	Debug header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __Debug_h
#define __Debug_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "String.hpp"
#include <stdarg.h>

//////////////////////////////////////////////////////////////////////////////
// TypeDefs
///////////////////////////////////////

typedef enum EDebugOutputType {
	OUTPUT_DEBUG,
	OUTPUT_NORMAL,
	OUTPUT_INFO,
	OUTPUT_WARN,
	OUTPUT_ERROR,
	OUTPUT_FATAL
};

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class Debug {
public:
	static void setDebugMessageHook(void HookFunc(const EDebugOutputType Type, const char * Text));
	static void setDebugMode(bool DebugMode);
	static void setDisableAllOutput(bool Disable);
	static bool getDebugMode();
	static String getLastOutput();

	static bool output(const char * TextToOutput, EDebugOutputType Type = OUTPUT_DEBUG, bool OutputToLog = false,
			   bool DateStamp = false, bool ForceOutput = false, ...);

	static bool out(const char * TextToOutput, ...);
	static bool outInfo(const char * TextToOutput, ...);
	static bool outWarn(const char * TextToOutput, ...);
	static bool outError(const char * TextToOutput, ...);
	static bool outFatal(const char * TextToOutput, ...);
	static bool outDebug(const char * TextToOutput, ...);

	Debug();
	virtual ~Debug();

private:
	static void (* m_HookFunc) (const EDebugOutputType, const char *);
	static bool m_DebugMode;
	static char * m_LogFile;
	static String m_LastOutput;
	static bool m_DisableAllOutput;
};

#endif // __Debug_h

