  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// DynamicArray.hpp
//
//	DynamicArray header file
//
///////////////////////////////////////
  ///////////////////////////////////////

/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef DynamicArray_h
#define DynamicArray_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "GarbageCollector.hpp"
#include <stdlib.h>

/////////////////////

template <class DataType> class DynamicArray : public GarbageCollectorManaged {
public:
	bool		deleteArray();
	bool		deleteElement(int Index);
	DataType * 	getArray();
	DataType &	getElementFromIndex(int Index);
	DataType *	getElementPtrFromIndex(int Index);
	int		getSize();
	bool		isIndexValid(int Index);
	bool		setSize(int Size);
		
	DataType &	operator [] (long Index);
		
	DynamicArray();
	virtual ~DynamicArray();

private:
	bool 		init();
	bool 		cleanUp();

	int 		m_Size;
	DataType *	m_Array;
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

template <class DataType> 
DynamicArray<DataType>::DynamicArray() {
	init();
}

template <class DataType>
DynamicArray<DataType>::~DynamicArray() {
	cleanUp();
}


template <class DataType>
bool DynamicArray<DataType>::init() {
	m_Size = 0;
	m_Array = NULL;

	return true;
}

template <class DataType>
bool DynamicArray<DataType>::cleanUp() {
	deleteArray();

	return true;
}

template <class DataType>
bool DynamicArray<DataType>::deleteArray() {
	if (m_Array) {
		GarbageCollector::free(m_Array);
		m_Array = NULL;
	}

	m_Size = 0;
	
	return true;
}

template <class DataType>
bool DynamicArray<DataType>::deleteElement(int Element) {
	if (!isIndexValid(Element))
		return false;

	for (int lp = Element; lp < (m_Size - 1); lp ++)
		m_Array[lp] = m_Array[lp + 1];

	return setSize(m_Size - 1);
}

template <class DataType>
DataType * DynamicArray<DataType>::getArray() {
	return m_Array;
}

template <class DataType>
DataType & DynamicArray<DataType>::getElementFromIndex(int Index) {
	return *getElementPtrFromIndex(Index);
}

template <class DataType>
DataType * DynamicArray<DataType>::getElementPtrFromIndex(int Index) {
	if (!isIndexValid(Index))
	{
		if (Index >= getSize())
		{
			if (!setSize(Index + 1))
				return NULL;
		}
	}
	
	return &m_Array[Index];
}

template <class DataType>
int DynamicArray<DataType>::getSize() {
	return m_Size;
}

template <class DataType>
bool DynamicArray<DataType>::isIndexValid(int Index) {
	if ((Index < 0) || (Index >= m_Size))
		return false;

	return true;
}

template <class DataType>
bool DynamicArray<DataType>::setSize(int Size) {
	if (Size == m_Size)
		return true;

	else if (Size == 0)
		return deleteArray();

	else if (Size < 0)
		return false;

	void * pRet = GarbageCollector::realloc(m_Array, sizeof(DataType) * Size);

	m_Size = Size;

	m_Array = (DataType *) pRet;

	return true;
}

template <class DataType>
DataType & DynamicArray<DataType>::operator [] (long Index) {
	return getElementFromIndex(Index);
}

#endif // DynamicArray_h
