////////////////////////////////////////////////////////////////////////////
//
// File.cpp
//
//	File main file
//
///////////////////////////////////////

/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <stdlib.h>
#include <string.h>

#include "FileIO.hpp"
#include "File.hpp"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

File::File() {
	m_File = NULL;
}

File::~File() {
	closeFile();
}

//////////////////////////////////////////////////////////////////////
// Class Body
//////////////////////////////////////////////////////////////////////

bool File::checkFile(const char * FileName, EFileOpen FileMode) {
	switch (FileMode)
	{
	case FILE_READ:
		if (!FileIO::doesFileExist(FileName))
			return false;
			
		break;
		
	case FILE_WRITE_OVERWRITE:
		if (!FileIO::touchRecursive(FileName, true))
			return false;			
			
		break;
		
	case FILE_WRITE_APPEND:
		if (!FileIO::doesFileExist(FileName))
			return false;
			
		break;
		
	case FILE_CREATE_NEW:
		if (!FileIO::touchRecursive(FileName, true))
			return false;			
		
		break;
		
	case FILE_CREATE_OPEN:
		if (!FileIO::touchRecursive(FileName, false))
			return false;			
			
		break;
		
	case FILE_CREATE_OVERWRITE:
		if (!FileIO::touchRecursive(FileName, true))
			return false;			
		
		break;
	
	case FILE_CREATE_APPEND:
		if (!FileIO::touchRecursive(FileName, false))
			return false;			
			
		break;
	}
	
	return true;
}

bool File::openFile(const char * FileName, EFileOpen FileMode, unsigned long FileSize) {
	if (FileName == "")
		return false;
		
	//if (!checkFile(FileName, FileMode))
	//	return false;
		
	closeFile();
		
	const char * OpenFlags;
	switch (FileMode) {
	default:
	case FILE_READ:
		OpenFlags = "r";
		break;
		
	case FILE_WRITE_OVERWRITE:
		OpenFlags = "w";
		break;
		
	case FILE_WRITE_APPEND:
		OpenFlags = "a";
		break;
		
	case FILE_CREATE_NEW:
		OpenFlags = "w";
		break;
		
	case FILE_CREATE_OPEN:
		OpenFlags = "r";
		break;
		
	case FILE_CREATE_OVERWRITE:
		OpenFlags = "w";
		break;
	
	case FILE_CREATE_APPEND:
		OpenFlags = "a";
		break;
	}
	
	m_File = fopen(FileName, OpenFlags);

	if (!m_File)
		return false;
		
	return true;
}
	
bool File::closeFile()
{
	if (m_File)
		fclose(m_File);
	else
		return false;

	m_FileName = "";
	m_File = NULL;

	return true;
}

int File::readLine(char * line, int BUFSIZE) {
	if (!m_File)
		return -1;
	
	char	tBuf[2];
	int 	len = 0;
	int	numread = 0;
					
	// input the file in raw mode using a buffer
	do {
		if ((numread = (int) fread(tBuf, sizeof(char), 1, m_File)) == 0)
			return -1;

		// check for new line
		if ( (tBuf[0] == 13) || (tBuf[0] == 10) ) {
			line[len] = '\0';
			return len;
		}
		
		line[len] = tBuf[0];
		len += numread;

		// check for overflow
		if (len >= BUFSIZE)
			return len;
	} while (numread != 0);
					
	line[len] = '\0';
	return len;
}

int File::read(const char * Buffer, int BufferSize, bool ConcatenateStr) {
	if (!m_File)
		return 0;
	
	//char	line[BufferSize + 1];
	char * line = new char[BufferSize + 1];
	int 	len = 0;
	int	numread = 0;
					
	// input the file in raw mode using a buffer
	do {
		numread = (int) fread(line, sizeof(char), BufferSize, m_File);
		line[numread] = '\0';
		
		if (ConcatenateStr)
			strcat((char *)Buffer, line);
		else
			strcpy((char *)Buffer, line);
		
		len += numread;
	} while (numread == BufferSize);

	delete [] line;
	return len;
}

bool File::readIntoStr(const char * Buffer, int BufferSize) {
	int	CharsRead;

	strcpy((char *)Buffer, "");
	do {
		CharsRead = read(Buffer, BufferSize, true);
	} while (CharsRead == BufferSize);

	return true;
}

int File::write(const char * Data, int DataLen) {
	if (!m_File)
		return 0;

	// output file
	return (int) fwrite(Data, sizeof(char), DataLen, m_File);
}

int File::write(const char * Text) {
	if (!m_File)
		return 0;

	return write(Text, strlen(Text));
}

int File::readIntoBuffer(const char * Buffer, int BufferSize) {
	if (!m_File) return 0;
	
	return (int) fread((char *)Buffer, sizeof(char), BufferSize, m_File);
}

const char * File::getFileName() {
	return m_FileName;
}

int File::getPosition() {
	return ftell(m_File);
}

bool File::seekTo(unsigned int BytePos) {
	return fseek(m_File, BytePos, SEEK_SET) != 0;
}
