  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// FileEventWatcher.cpp
//
//	FileEventWatcher class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "FileEventWatcher.hpp"
#include "Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Overridables
///////////////////////////////////////
	
////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

FileEventWatcher::FileEventWatcher() {
	m_FD = -1;
}

FileEventWatcher::~FileEventWatcher() {
	setFileDescriptor(-1);
}

////////////////////////////////////////////////////////////////////////////
// Overridables
///////////////////////////////////////

bool FileEventWatcher::checkEventsWaiting() {
	if (m_FD == -1)
		return false;
		
	struct input_event ibuffer[IBUF_SIZE];
	int 		r, events, i;
	fd_set 		readfds;
	struct timeval	TimeOut;
	bool		ret = false;

	TimeOut.tv_sec = 0;
	TimeOut.tv_usec = 100000;
		
	// tell select which sockets to poll
	FD_ZERO(&readfds);
	FD_SET(m_FD, &readfds);

	// wait on sockets
	int NumSelected = select(m_FD + 1, &readfds, NULL, NULL, &TimeOut);
	if (NumSelected == -1)
		continue;
	else if (NumSelected == 0) // timeout
		continue;
		
	if (FD_ISSET(m_FD, &readfds)) {
		// read event!
		r = read(m_FD, ibuffer, sizeof(struct input_event) * IBUF_SIZE);
		if (r > 0) {
			ret = true;
			events = r / sizeof(struct input_event);
			for (i = 0; i < events; i ++)
				processEvent(&ibuffer[i]);
		} else {
			Debug::outError("FileEventWatcher :: Failed to Read from Device!: %s", strerror(errno));
		}
	}
		
	return ret;
}


void FileEventWatcher::setFileDescriptor(int FD) {
	m_FD = FD;
}
