  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// PluginLoader.cpp
//
//	PluginLoader class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "FileIO.hpp"
#include "String.hpp"
#include "Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

template <class Plugin>
PluginLoader<Plugin>::PluginLoader() {
	m_pPlugin = NULL;
}

template <class Plugin>
PluginLoader<Plugin>::~PluginLoader() {
	closePlugin();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

/*
 * Create a new plugin, by dynamically loading the given file
 */
template <class Plugin>
Plugin * PluginLoader<Plugin>::loadPlugin(const char * PluginName, LinkList<void> * pArgList) {
	// first try users home dir
	// then try PACKAGE_PREFIX/share/gizmo
	// then try devel dirs (./)
	int ArrNum = 0;

	const int PATHS_TO_CHECK = 14;
	String Paths[PATHS_TO_CHECK];

	// try developer dir
	Paths[ArrNum ++] = String("./plugins/scriptingPlugins/python/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/powermate/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/alsaMixer/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/xosd/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/generic/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/soundVisualization/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/cpuUsage/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/x11/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/xmmsControl/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/bmpControl/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/lirc/.libs/lib") + PluginName + ".so";
	Paths[ArrNum ++] = String("./plugins/pulse/.libs/lib") + PluginName + ".so";
	
	// try home dir
	char * UserHome = getenv("HOME");
	if (!UserHome)
		Debug::outWarn("PluginLoader :: Warning! $HOME not defined!");
	else
		Paths[ArrNum ++] = UserHome + String("/.gizmo/plugins/lib") + PluginName + ".so";
	
	// then try prefix/share/plugins/
	Paths[ArrNum ++] = PACKAGE_PREFIX + String("/share/gizmo/plugins/lib") + PluginName + ".so";

	// now check them in order
	int lp;
	for (lp = 0; lp < PATHS_TO_CHECK; lp ++) {
		Debug::outDebug("PluginLoader :: loadPlugin [%s]", Paths[lp].getString());
		if (FileIO::doesFileExist(Paths[lp]))
			if (openLibrary(Paths[lp]))
				break;
	}
	if (lp == PATHS_TO_CHECK)
		return NULL;

	// attach the necessary functions
	Plugin * (* getPlugin) (LinkList<void> *) = (Plugin * (*) (LinkList<void> *)) getFunction("getPlugin");
	if (!getPlugin) {
		Debug::outDebug("PluginLoader :: Failed to Load Shared Library [%s]", PluginName);
		return NULL;
	}

	if ((m_pPlugin = getPlugin(pArgList)) == NULL) {
		Debug::outDebug("PluginLoader :: Failed to Load Plugin [%s]", PluginName);
		return NULL;
	}
	
	// set plugin name
	m_pPlugin->setPluginName(PluginName);
	
	// plugin is loaded!
	if (!m_pPlugin->onPluginOpened()) {
		Debug::outDebug("PluginLoader :: Failed to Initialize Plugin [%s]", PluginName);
		return NULL;
	}
	
	return m_pPlugin;
}

/*
 * Create a new plugin, by dynamically loading the given file
 */
template <class Plugin>
void PluginLoader<Plugin>::closePlugin() {
	if (m_pPlugin) {
		m_pPlugin->closePlugin();
		m_pPlugin = NULL;
	}
	closeLibrary();
}
