  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// ScriptFunction.hpp
//
//	ScriptFunction header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __ScriptFunction_h
#define __ScriptFunction_h

#include "LinkList.hpp"
#include "String.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

typedef enum {
	SCRIPT_ARG_NUMBER,
	SCRIPT_ARG_STRING,
	SCRIPT_ARG_BOOLEAN
} ScriptFuncArgType;

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class ScriptFuncArg {
public:
	ScriptFuncArgType	Type;
	double			ArgNumber;
	String			ArgString;
	bool			ArgBoolean;
	
	void			setBoolean(bool Value);
	void			setNumber(double Value);
	void			setString(String Value);

	ScriptFuncArg();
	ScriptFuncArg(String Value);
	ScriptFuncArg(double Value);
	virtual ~ScriptFuncArg();
};

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class ScriptFunction {
public:
	LinkList<ScriptFuncArg> 	InArgList;
	LinkList<ScriptFuncArg> 	OutArgList;
	void *				Arguments;
	String				FunctionName;
	String				FunctionDescription;

	ScriptFuncArg *			addInArg(ScriptFuncArgType Type);
	ScriptFuncArg *			addInArg(ScriptFuncArgType Type, String Value);
	ScriptFuncArg *			addInArg(ScriptFuncArgType Type, double Value);
	ScriptFuncArg *			addOutArg(ScriptFuncArgType Type);
	ScriptFuncArg *			addOutArg(ScriptFuncArgType Type, String Value);
	ScriptFuncArg *			addOutArg(ScriptFuncArgType Type, double Value);
	void				debugPrintInArgs();
	virtual bool			executeFunction();
	ScriptFuncArg *			getInArg(int Index);
	ScriptFuncArg *			getOutArg(int Index);
	void				resetArgLists();
	
	void (* getFunction(void)) (ScriptFunction *); // return a function pointer

	ScriptFunction();
	ScriptFunction(void (* Func) (ScriptFunction *), String FuncName, void * Args);
	virtual ~ScriptFunction();

	static bool 			checkScriptFunc(ScriptFunction * pFunc);
	
private:		
	void 				resetArgList(LinkList<ScriptFuncArg> * ArgList);
	
	void (* m_Function) (ScriptFunction * pFunc);
};

#endif // __ScriptFunction_h

