  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// SharedMemorySegment.hpp
//
//	SharedMemorySegment header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __SharedMemorySegment_h
#define __SharedMemorySegment_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include "Average.hpp"
#include "String.hpp"
#include "EventWatcherInterface.hpp"

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class SharedMemorySegment {
public:
	bool		checkEventsWaiting();
	void * 		getSharedMemorySegment();
	int		getShmPollDelay();
	bool		setShmDataString(const char * data);
	void		setShmPollDelay(int Delay);
	bool 		signalShmChange();
	void		startMonitoringSegmentForChanges();

	virtual void	onSharedMemorySegmentChange(void * data);
	
	SharedMemorySegment(int ID, unsigned int DataSize, bool HardClose = true);
	virtual ~SharedMemorySegment();
	
private:
	void		closeSHM();
	bool 		initSHM();
	void 		updateSegmentInfo();
	
	int		m_Delay;
	void *		m_Data;
	int		m_ID;
	bool		m_HardClose;
	int		m_DataSize;
	String		m_CurContents;

	key_t		m_SHMkey;
	int		m_SHMid;
	shmid_ds 	m_ShmDS;
};

#endif // __SharedMemorySegment_h
