  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// String.hpp
//
//	String header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __String_h
#define __String_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class String {
public:
	////////////////////////////////////////////
	// Public Functions
	////////////////////////////////////////////

	bool	      beginsWith(String CompareString, bool CaseSensitive = true);
	bool          clearString();
	int           compareWithString(const String & CompareString, bool CaseSensitive = true);
	int           compareWithString(const char * CompareString, bool CaseSensitive = true);
	int           compareWithStringInsensitive(const String & CompareString);
	int           compareWithStringInsensitive(const char * CompareString);
	bool          concatenate(const char * stringToAdd);
	bool          concatenate(const String & stringToAdd);
	bool          concatenate(const String & Str1, const String & Str2);
	bool          concatenate(const char * Str1, const String & Str2);
	long          convertToLong();
	int           convertToInt();
	unsigned long convertToULONG();
	float         convertToFloat();
	double        convertStringToNumericalValue(bool CaseSensitive = true);
	bool	      endsWith(String CompareString, bool CaseSensitive = true);
	int           find(const char * StringToFind, bool CaseSensitive = true);
	int           find(const char * StringToFind, int StartPos, bool CaseSensitive = true);
	int           find(const String StringToFind, bool CaseSensitive = true);
	int           find(const String StringToFind, int StartPos, bool CaseSensitive = true);
	int           findReverse(const char * StringToFind);
	int           findReverse(const String StringToFind);
	int           findInsensitive(const char * StringToFind, int StartPos = 0);
	int           findInsensitive(const String StringToFind, int StartPos = 0);
	bool          format(char * Text, ...);
	bool          freeString();
	const char    getCharAt(int Index);
	int           getLength();
	const char *  getString() { return (const char *) m_String; };
	void          initToCharFromInt(int IntegerValueOfChar);
	bool          isEmpty();
	bool          isNumeric();
	bool          lTrim();
	String        mid(int StartPos, int EndPos = -1);
	bool          reAllocToSize(int Size);
	void 	      removeCR();
	void          replaceSubstring(const char * StringToReplace, const String ReplaceWith, bool CaseSensitive = true);
	void          replaceSubstring(const String StringToReplace, const char * ReplaceWith, bool CaseSensitive = true);
	void          replaceSubstring(const char * StringToReplace, const char * ReplaceWith, bool CaseSensitive = true);
	void          replaceSubstring(const String StringToReplace, const String ReplaceWith, bool CaseSensitive = true);
	bool          rTrim();
	bool          setCharAt(int Index, char Char);
	bool          setValue(const char data);
	bool          setValue(const char * data);
	bool          setValue(const String & Str);
	bool          setValueFromChar(char CharToSet);
	void          setToLowerCase();
	void          setToUpperCase();
	int           stringToInt(const String String);
	String        toLower();
	String        toUpper();
	bool          trim();

	////////////////////////////////////////////
	// Construction
	////////////////////////////////////////////
	
	String();
	String(const char * initVal);
	String(const String & initVal);
	String(const char initVal);
	virtual ~String();

	////////////////////////////////////////////
	// Local Overloads
	////////////////////////////////////////////
	
	operator const char * () { return (const char *) m_String; }
	operator const String () { return *this; }
	operator const String * () { return this; }

	const String & operator = (const String & initVal);
	const String & operator = (const char * initVal);
	const String & operator = (const char initVal);

	bool operator == (const String & equalityTest);
	bool operator == (const char * equalityTest);

	bool operator != (const String & equalityTest);
	bool operator != (const char * equalityTest);
	
	String & operator += (const String & concatVal);
	String & operator += (const char * concatVal);
	String & operator += (const char concatVal);
	
	const char operator [] (int Index);

	////////////////////////////////////////////
	// Global Overloads
	////////////////////////////////////////////
	
	friend const String operator + (const String & String1, const String & String2);

private:
	////////////////////////////////////////////
	// Private Member Variables
	////////////////////////////////////////////
	
	char * 		m_String;
	int		m_StrLen;
	static int 	m_FloatPrecision;

	////////////////////////////////////////////
	// Private Functions
	////////////////////////////////////////////

	void init();
};

#endif // __String_h

