  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// Tokenizer.cpp
//
//	Tokenizer class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "Tokenizer.hpp"
#include "Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

Tokenizer::Tokenizer(const char * Token, const char * Delimeter) {
	tokenize(Token, Delimeter);
}

Tokenizer::~Tokenizer() {
 	resetTokenList();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void Tokenizer::resetTokenList() {
	if (getSize() == 0)
		return;

	moveToHead();
	do {
		String * pToken = getData();
		pToken->freeString();
		delete pToken;
	} while (moveNext());

	deleteAll();
}

void Tokenizer::tokenize(const char * Token, const char * Delimeter, bool RemoveWhiteSpace) {
	resetTokenList();
	
	int curPos = 0;
	int dPos;
	String Tokenee = Token;
	do {
		dPos = Tokenee.find(Delimeter, curPos);
		if (dPos < 0)
			dPos = Tokenee.getLength();

		String * pToken = new String(Tokenee.mid(curPos, dPos - 1));
		addTail(pToken);
		if (RemoveWhiteSpace)
			pToken->trim();

		curPos = dPos + 1;
	} while ( (dPos >= 0) && (curPos < Tokenee.getLength()) );
}

void Tokenizer::debugPrint() {
	if (getSize() == 0)
		return;

	moveToHead();
	do {
		Debug::outDebug("Token [%d] == [%s]", getPosition(), getData()->getString());
	} while (moveNext());
}
