  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginAlsaMixer.hpp
//
//	GizmoPluginAlsaMixer header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginAlsaMixer_h
#define __GizmoPluginAlsaMixer_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/Debug.hpp"
#include "AlsaMixerInterface.hpp"
#include "ScriptFunctionOnAlsaMixerEvent.hpp"
#include "ScriptFunctionAlsaMixerGetVolume.hpp"
#include "ScriptFunctionAlsaMixerSetVolume.hpp"
#include "ScriptFunctionAlsaMixerGetMute.hpp"
#include "ScriptFunctionAlsaMixerSetMute.hpp"
#include "ScriptFunctionAlsaMixerGetMixerName.hpp"
#include <alsa/asoundlib.h>

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginAlsaMixer : public AlsaMixerInterface {
public:
	bool		checkEventsWaiting();
	void		closeMixer();
	void 		closePlugin();
	bool		initPlugin();
	bool 		decrementVolumePercent(float Value);
	bool 		decrementVolumeReal(long Value);
	bool 		openMixer(const char * Card, const char * Mixer);
	String		getDeviceName();
	String		getDeviceNameAlt();
	virtual snd_mixer_elem_t * getMixerElem();
	virtual int	getSwitch();
	virtual float	getVolumePercent();
	virtual long 	getVolume();
	virtual void 	getVolumeFromCard();
	bool 		incrementVolumePercent(float Value);
	bool 		incrementVolumeReal(long Value);
	void 		onActiveSenderChange(bool NowActive);
	virtual void 	onMixerEvent(float VolumePercent, bool Mute);
	virtual bool	onPluginOpened();
	virtual bool 	setSwitch(int Value);
	virtual bool 	setVolumePercent(float Percent);
	bool 		setVolume(long Value);
	bool 		toggleSwitch();

	GizmoPluginAlsaMixer(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginAlsaMixer();
	
	static int 	mixerCallbackRefresh(snd_mixer_elem_t * elem, unsigned int mask);

private:
	void 		closeAlsa();
	int		parseSimpleID();
	
	ScriptEngineInterface * m_pScriptEngine;

	String		m_DeviceCard;
	String		m_DeviceMixer;
	bool		m_WatchingDevice;
	
	String		m_Card;
	String		m_Mixer;
	int		m_ElemCount;
	snd_mixer_t *	m_Handle;
	snd_mixer_elem_t * m_Element;
	snd_mixer_selem_id_t * m_SID;
	snd_ctl_card_info_t * m_HWInfo;
	snd_ctl_t *	m_CTLHandle;
	long		m_VolumeMin;
	long		m_VolumeMax;
	long		m_VolumeCur;
	float		m_VolumePercent;
	int		m_HasSwitch;
	int		m_Switch;
	
	ScriptFunctionOnAlsaMixerEvent 		m_ScriptFuncOnAlsaMixerEvent;
	ScriptFunctionAlsaMixerGetVolume	m_ScriptFuncAlsaMixerGetVolume;
	ScriptFunctionAlsaMixerSetVolume	m_ScriptFuncAlsaMixerSetVolume;
	ScriptFunctionAlsaMixerGetMute		m_ScriptFuncAlsaMixerGetMute;
	ScriptFunctionAlsaMixerSetMute		m_ScriptFuncAlsaMixerSetMute;
	ScriptFunctionAlsaMixerGetMixerName	m_ScriptFuncAlsaMixerGetMixerName;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginAlsaMixer :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginAlsaMixer :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginAlsaMixer(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginAlsaMixer_h

