  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// BMPControl.cpp
//
//	BMPControl class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution.
    * Neither the name of TimeSync nor the names of its contributors
      may be used to endorse or promote products derived from this software 
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "BMPControl.hpp"

#include "libGizmo/Debug.hpp"
#include <unistd.h>
#include <bmp/beepctrl.h>

////////////////////////////////////////////////////////////////////////////
// Globals, Defines, Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overloads
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

BMPControl::BMPControl() {
	m_Session = -1;
	m_Launch = false;
}

BMPControl::~BMPControl() {
	closeBMPControl();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

bool BMPControl::initBMPControl(bool Launch) {
	// check if already connected / BMP running
	if ( (m_Session == -1) || (!xmms_remote_is_running(m_Session)) ) {
		// try to find automatically
		for (m_Session = 0; m_Session < 16; m_Session ++)
			if (xmms_remote_is_running(m_Session))
				break;

		if (m_Session == 16) {
			closeBMPControl();
			return false;
		}
	}

	// we have a valid BMP session!

	return true;
}

void BMPControl::closeBMPControl() {
	m_Session = -1;
}

bool BMPControl::BMPPreviousTrack() {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_playlist_prev(m_Session);

	return true;
}

bool BMPControl::BMPNextTrack() {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_playlist_next(m_Session);

	return true;
}

bool BMPControl::BMPSetTrack(int Track) {
	if ( (!initBMPControl(m_Launch)) || (Track < 0) )
		return false;

	xmms_remote_set_playlist_pos(m_Session, Track);

	return true;
}

bool BMPControl::BMPPlay() {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_play(m_Session);

	return true;
}

bool BMPControl::BMPPause() {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_pause(m_Session);

	return true;
}

bool BMPControl::BMPStop() {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_stop(m_Session);

	return true;
}

bool BMPControl::BMPIsPlaying() {
	if (!initBMPControl(m_Launch))
		return false;

	return xmms_remote_is_playing(m_Session);
}

bool BMPControl::BMPIsPaused() {
	if (!initBMPControl(m_Launch))
		return false;

	return xmms_remote_is_paused(m_Session);
}

int BMPControl::BMPGetPlaylistPosition() {
	if (!initBMPControl(m_Launch))
		return false;

	return xmms_remote_get_playlist_pos(m_Session);
}

bool BMPControl::BMPSeek(int Offset) {
	if (!initBMPControl(m_Launch))
		return false;

	xmms_remote_jump_to_time(m_Session, Offset);
	return true;
}

int BMPControl::BMPGetOutputTime() {
	if (!initBMPControl(m_Launch))
		return -1;

	return xmms_remote_get_output_time(m_Session);
}

