  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionPowerMateSetLEDPercent.cpp
//
//	ScriptFunctionPowerMateSetLEDPercent class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionPowerMateSetLEDPercent.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionPowerMateSetLEDPercent::executeFunction() {
	//Debug::outDebug("ScriptFunctionPowerMateSetLEDPercent :: executeFunction");
	
	ScriptFuncArg * pInArg1 = getInArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionPowerMateGetLED :: executeFunction -- No In Parameter!");
		return false;
	}
	
	ScriptFuncArg * pInArg2 = getInArg(1);
	if (!pInArg2) {
		Debug::out("ScriptFunctionPowerMateSetLEDPercent :: executeFunction -- No Out Parameter!");
		return false;
	}
	
	// find device
	LinkList<PowerMateInterface> * pPluginList = PowerMateInterface::getPluginList();
	if (pPluginList->getSize() == 0) {
		Debug::out("ScriptFunctionPowerMateGetLED :: PluginList Badness!");
		return false;
	}
	pPluginList->moveToHead();
	do {
		PowerMateInterface * pInt = pPluginList->getData();
		if (pInt->getDeviceID() == pInArg1->ArgNumber) {
			float Percent = pInArg2->ArgNumber;
			Percent /= 100.0f;
			float Value = 255.0f * Percent;
			pInt->onPowerMateSetLED((int) Value);
			break;
		}
	} while (pPluginList->moveNext());
		
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionPowerMateSetLEDPercent::ScriptFunctionPowerMateSetLEDPercent(PowerMateInterface * pPlugin) {
	m_pPlugin = pPlugin;
	//FunctionName.format("powerMateSetLED_Dev%d", pPlugin->getDeviceID());
	FunctionName.format("powerMateSetLEDPercent");
	FunctionDescription = "Sets the LED Level of the PowerMate in Percentage";
	
	addInArg(SCRIPT_ARG_NUMBER);		// IN  DeviceID
	addInArg(SCRIPT_ARG_NUMBER);		// IN  LED Level
}

ScriptFunctionPowerMateSetLEDPercent::~ScriptFunctionPowerMateSetLEDPercent() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////
