  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginSoundVisualization.cpp
//
//	GizmoPluginSoundVisualization class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginSoundVisualization.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/UtilTime.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

#define SHARED_MEM_ID	303
#define SHARED_MEM_SIZE	sizeof(unsigned char) * 5

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool GizmoPluginSoundVisualization::onPluginOpened() {
	Debug::outDebug("GizmoPluginSoundVisualization :: onPluginOpened");
	
	return initPlugin();
}

void GizmoPluginSoundVisualization::onSoundVisualization(unsigned char FrameID, unsigned char LeftChannel, unsigned char RightChannel, unsigned char Combined, unsigned char Special) {
	ScriptFuncArg * pInArg1 = m_ScriptFuncOnSoundVisualization.getInArg(0);
	if (!pInArg1) {
		Debug::outDebug("GizmoPluginSoundVisualization :: Failed to Call OnSoundVisualization -- Arg 1");
		return;
	}
	ScriptFuncArg * pInArg2 = m_ScriptFuncOnSoundVisualization.getInArg(1);
	if (!pInArg2) {
		Debug::outDebug("GizmoPluginSoundVisualization :: Failed to Cll OnSoundVisualization -- Arg 2");
		return;
	}
	ScriptFuncArg * pInArg3 = m_ScriptFuncOnSoundVisualization.getInArg(2);
	if (!pInArg3) {
		Debug::outDebug("GizmoPluginSoundVisualization :: Failed to Call OnSoundVisualization -- Arg 3");
		return;
	}
	ScriptFuncArg * pInArg4 = m_ScriptFuncOnSoundVisualization.getInArg(3);
	if (!pInArg4) {
		Debug::outDebug("GizmoPluginSoundVisualization :: Failed to Call OnSoundVisualization -- Arg 4");
		return;
	}
	ScriptFuncArg * pInArg5 = m_ScriptFuncOnSoundVisualization.getInArg(4);
	if (!pInArg5) {
		Debug::outDebug("GizmoPluginSoundVisualization :: Failed to Call OnSoundVisualization -- Arg 4");
		return;
	}
	
	pInArg1->ArgNumber = FrameID;
	pInArg2->ArgNumber = LeftChannel;
	pInArg3->ArgNumber = RightChannel;
	pInArg4->ArgNumber = Combined;
	pInArg5->ArgNumber = Special;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnSoundVisualization);
}

bool GizmoPluginSoundVisualization::checkEventsWaiting() {
	static unsigned char LastIndex = 255;
	
	if (!m_pSharedMem)
		return false;
	unsigned char * VisData = (unsigned char *) m_pSharedMem->getSharedMemorySegment();	
	if (VisData == NULL) {
		m_Watching = false;
		return false;
	}
	if (LastIndex == VisData[0])
		return false;
		
	// data has changed!
	LastIndex = VisData[0];
	onSoundVisualization(VisData[0], VisData[1], VisData[2], VisData[3], VisData[4]);
	
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginSoundVisualization::GizmoPluginSoundVisualization(ScriptEngineInterface * pScriptEngine, const char * DevicePath) {
	Debug::outDebug("GizmoPluginSoundVisualization :: SoundVisualization Input Plugin Loading [%s]...", DevicePath);
	
	m_pScriptEngine = pScriptEngine;
	m_DevicePath = DevicePath;
	m_pSharedMem = NULL;
}

GizmoPluginSoundVisualization::~GizmoPluginSoundVisualization() {
	closePlugin();
	Debug::outDebug("GizmoPluginSoundVisualization :: SoundVisualization Input Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginSoundVisualization::closePlugin() {
	Debug::outDebug("GizmoPluginSoundVisualization :: SoundVisualization Input Plugin Closing...");
	
	m_Watching = false;
	
	if (m_pSharedMem)
		delete m_pSharedMem;
}

bool GizmoPluginSoundVisualization::initPlugin() {
	Debug::outDebug("    SoundVisualization Input Plugin v%s [%s]", PACKAGE_VERSION, m_DevicePath.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginSoundVisualization :: Invalid Script Engine!");
		return false;
	}
	
	// open the device
	m_pSharedMem = new SharedMemorySegment(SHARED_MEM_ID, SHARED_MEM_SIZE);
	if ( (m_pSharedMem == NULL) || (m_pSharedMem->getSharedMemorySegment() == NULL) )
		return false;
		
	// register script functions
	Debug::outDebug("GizmoPluginSoundVisualization :: Registering Functions...");
	
	Debug::out("\t- Found [%s] at [%s]", m_PluginName.getString(), m_DevicePath.getString());	
	Debug::outDebug("GizmoPluginSoundVisualization :: SoundVisualization Input Plugin Initialized.");
	return true;
}

String GizmoPluginSoundVisualization::getDeviceName() {
	return m_DeviceName;
}

String GizmoPluginSoundVisualization::getDeviceNameAlt() {
	return m_DevicePath;
}
