  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginSoundVisualization.hpp
//
//	GizmoPluginSoundVisualization header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2006, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginSoundVisualization_h
#define __GizmoPluginSoundVisualization_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/SharedMemorySegment.hpp"
#include "SoundVisualizationInterface.hpp"
#include "ScriptFunctionOnSoundVisualization.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginSoundVisualization : public SoundVisualizationInterface {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual bool	onPluginOpened();
	virtual void 	onSoundVisualization(unsigned char FrameID, unsigned char LeftChannel, unsigned char RightChannel, unsigned char Combined, unsigned char Special);
	
	GizmoPluginSoundVisualization(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginSoundVisualization();

private:
	ScriptEngineInterface * m_pScriptEngine;
	String		m_DevicePath;
	String		m_DeviceName;
	String		m_DeviceNameAlt;
	
	SharedMemorySegment *	m_pSharedMem;
	ScriptFunctionOnSoundVisualization m_ScriptFuncOnSoundVisualization;

	bool		m_Watching;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginSoundVisualization :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginSoundVisualization :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginSoundVisualization(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginSoundVisualization_h
