  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginX11.hpp
//
//	GizmoPluginX11 header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginX11_h
#define __GizmoPluginX11_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/String.hpp"
#include "X11Interface.hpp"
#include "ScriptFunctionOnX11ForegroundWindowChange.hpp"
#include "ScriptFunctionX11GetCurrentWorkspace.hpp"
#include "ScriptFunctionX11GetNumWorkspaces.hpp"
#include "ScriptFunctionX11SetWorkspace.hpp"
#include "ScriptFunctionX11SetWorkspaceNext.hpp"
#include "ScriptFunctionX11SetWorkspacePrev.hpp"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginX11 : public X11Interface {
public:
	bool		checkEventsWaiting();
	void		closeDisplay();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool 		getForegroundWindowProperties(String * WindowName);
	unsigned int 	getKeyboardModifierState(bool * shift, bool * alt, bool * ctrl);
	virtual int	getNumWorkspaces();
	String		getWindowName(Window window, bool recurse = true);
	virtual int	getWorkspace();
	virtual bool	setWorkspace(int Workspace);
	bool		initPlugin();
	virtual void 	onForegroundWindowChange(String NewWindow);
	bool 		openDisplay(const char * DisplayName);
	virtual bool	onPluginOpened();
	//bool 		sendKey(int KeyCode, bool Ctrl, bool Alt, bool Shift);
	//bool 		sendKeyXTest(int KeyCode, bool Ctrl, bool Alt, bool Shift);
	//bool 		sendMouseButton(int button, bool Pressed);
	virtual bool	setWorkspaceNext();
	virtual bool	setWorkspacePrev();

	GizmoPluginX11(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginX11();
	
	//static void 	scriptFunction_X11GetActiveWinTitle(ScriptFunction * pFunc);
	//static void 	scriptFunction_X11GetKeyMods(ScriptFunction * pFunc);
	static int	x11ErrorHandler(Display * display, XErrorEvent * error);
	static int	x11IOErrorHandler(Display * display);

private:
	void 		ewmh_set_current_desktop (int workspace);
	void 		ewmh_get_desktop_info(struct gnome_workspace_info *info);
	void 		get_vdesktop_info(int * column, int * row, int * n_columns, int * n_rows);
	int 		Get_Window_Property(Display *display, Window target_win, Atom atom, long *length, Atom *type, int *size, char **data);
	int 		send_client_message_32 (Display *display, Window recipient, Window event_window, Atom message_type, long event_mask, int n_longs, ...);
	
	ScriptEngineInterface * m_pScriptEngine;

	bool		m_WatchingDevice;
	Display	*	m_Display;
	int		m_Screen;
	Window		m_RootWindow;
	String		m_DisplayName;
	String		m_DevicePath;
	
	ScriptFunctionOnX11ForegroundWindowChange	m_ScriptFuncOnX11ForegroundWindowChange;

	ScriptFunctionX11GetCurrentWorkspace		m_ScriptFuncX11GetCurrentWorkspace;
	ScriptFunctionX11GetNumWorkspaces		m_ScriptFuncX11GetNumWorkspaces;
	ScriptFunctionX11SetWorkspace			m_ScriptFuncX11SetWorkspace;
	ScriptFunctionX11SetWorkspaceNext		m_ScriptFuncX11SetWorkspaceNext;
	ScriptFunctionX11SetWorkspacePrev		m_ScriptFuncX11SetWorkspacePrev;
	
	static bool	m_ErrorHandlerHit;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginGeneric :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginGeneric :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginX11(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginX11_h

