  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginXMMSControl.hpp
//
//	GizmoPluginXMMSControl header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginXMMSControl_h
#define __GizmoPluginXMMSControl_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/Debug.hpp"
#include "XMMSControlInterface.hpp"
#include "XMMSControl.hpp"
#include "ScriptFunctionXMMSPlay.hpp"
#include "ScriptFunctionXMMSGetOutputTime.hpp"
#include "ScriptFunctionXMMSGetPlaylistPosition.hpp"
#include "ScriptFunctionXMMSIsPaused.hpp"
#include "ScriptFunctionXMMSIsPlaying.hpp"
#include "ScriptFunctionXMMSNextTrack.hpp"
#include "ScriptFunctionXMMSPause.hpp"
#include "ScriptFunctionXMMSPlay.hpp"
#include "ScriptFunctionXMMSPreviousTrack.hpp"
#include "ScriptFunctionXMMSSeek.hpp"
#include "ScriptFunctionXMMSSetTrack.hpp"
#include "ScriptFunctionXMMSStop.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////
	
//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginXMMSControl : public XMMSControlInterface, public XMMSControl {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual bool	onPluginOpened();
	virtual int 	xmmsGetPlaylistPosition();
	virtual int	xmmsGetOutputTime();
	virtual bool	xmmsIsPaused();
	virtual bool	xmmsIsPlaying();
	virtual bool	xmmsNextTrack();
	virtual bool	xmmsPause();
	virtual bool	xmmsPlay();
	virtual bool	xmmsPreviousTrack();
	virtual bool	xmmsSeek(int Offset);
	virtual bool	xmmsSetTrack(int Track);
	virtual bool	xmmsStop();

	GizmoPluginXMMSControl(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginXMMSControl();
		
private:
	ScriptEngineInterface * m_pScriptEngine;

	ScriptFunctionXMMSPlay		m_ScriptFuncXMMSPlay;
	ScriptFunctionXMMSGetOutputTime	m_ScriptFuncXMMSGetOutputTime;
	ScriptFunctionXMMSGetPlaylistPosition m_ScriptFuncXMMSGetPlaylistPosition;
	ScriptFunctionXMMSIsPaused	m_ScriptFuncXMMSIsPaused;
	ScriptFunctionXMMSIsPlaying	m_ScriptFuncXMMSIsPlaying;
	ScriptFunctionXMMSNextTrack	m_ScriptFuncXMMSNextTrack;
	ScriptFunctionXMMSPause		m_ScriptFuncXMMSPause;
	ScriptFunctionXMMSPreviousTrack	m_ScriptFuncXMMSPreviousTrack;
	ScriptFunctionXMMSSeek		m_ScriptFuncXMMSSeek;
	ScriptFunctionXMMSSetTrack	m_ScriptFuncXMMSSetTrack;
	ScriptFunctionXMMSStop 		m_ScriptFuncXMMSStop;

};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginXMMSControl(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginXMMSControl_h

