  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// XMMSControl.cpp
//
//	XMMSControl class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution.
    * Neither the name of TimeSync nor the names of its contributors
      may be used to endorse or promote products derived from this software 
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "XMMSControl.hpp"

#include "libGizmo/Debug.hpp"
#include <unistd.h>
#include <xmms/xmmsctrl.h>

////////////////////////////////////////////////////////////////////////////
// Globals, Defines, Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overloads
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

XMMSControl::XMMSControl() {
	m_Session = -1;
	m_Launch = false;
}

XMMSControl::~XMMSControl() {
	closeXMMSControl();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

bool XMMSControl::initXMMSControl(bool Launch) {
	// check if already connected / xmms running
	if ( (m_Session == -1) || (!xmms_remote_is_running(m_Session)) ) {
		// try to find automatically
		for (m_Session = 0; m_Session < 16; m_Session ++)
			if (xmms_remote_is_running(m_Session))
				break;

		if (m_Session == 16) {
			closeXMMSControl();
			return false;
		}
	}

	// we have a valid xmms session!
	return true;
}

void XMMSControl::closeXMMSControl() {
	m_Session = -1;
}

bool XMMSControl::XMMSPreviousTrack() {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_playlist_prev(m_Session);
	return true;
}

bool XMMSControl::XMMSNextTrack() {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_playlist_next(m_Session);
	return true;
}

bool XMMSControl::XMMSSetTrack(int Track) {
	if ( (!initXMMSControl(m_Launch)) || (Track < 0) )
		return false;

	xmms_remote_set_playlist_pos(m_Session, Track);
	return true;
}

bool XMMSControl::XMMSPlay() {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_play(m_Session);
	return true;
}

bool XMMSControl::XMMSPause() {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_pause(m_Session);
	return true;
}

bool XMMSControl::XMMSStop() {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_stop(m_Session);
	return true;
}

bool XMMSControl::XMMSIsPlaying() {
	if (!initXMMSControl(m_Launch))
		return false;

	return xmms_remote_is_playing(m_Session);
}

bool XMMSControl::XMMSIsPaused() {
	if (!initXMMSControl(m_Launch))
		return false;

	return xmms_remote_is_paused(m_Session);
}

int XMMSControl::XMMSGetPlaylistPosition() {
	if (!initXMMSControl(m_Launch))
		return false;

	return xmms_remote_get_playlist_pos(m_Session);
}

bool XMMSControl::XMMSSeek(int Offset) {
	if (!initXMMSControl(m_Launch))
		return false;

	xmms_remote_jump_to_time(m_Session, Offset);
	return true;
}

int XMMSControl::XMMSGetOutputTime() {
	if (!initXMMSControl(m_Launch))
		return -1;

	return xmms_remote_get_output_time(m_Session);
}
