  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginXOSD.cpp
//
//	GizmoPluginXOSD class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginXOSD.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

#define MAX_XOSD_LINES		2
#define DEFAULT_FONT_SIZE	30
#define DEFAULT_FONT		"Arial"
#define DEFAULT_FONT_COLOR	"Cyan"
#define SLIDER_LINE		1
#define TEXT_BUFSIZE		1024

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool GizmoPluginXOSD::onPluginOpened() {
	Debug::outDebug("GizmoPluginXOSD :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginXOSD::GizmoPluginXOSD(ScriptEngineInterface * pScriptEngine, const char * DevicePath) : 
	XOSD(MAX_XOSD_LINES),
	m_ScriptFuncXOSDSetFontName(this),
	m_ScriptFuncXOSDSetFontColor(this),
	m_ScriptFuncXOSDSetFontSize(this),
	m_ScriptFuncXOSDDrawSlider(this),
	m_ScriptFuncXOSDDrawPercentage(this),
	m_ScriptFuncXOSDPrintString(this)
{
	Debug::outDebug("GizmoPluginXOSD :: XOSD Input Plugin Loading [%s]...", DevicePath);
	
	m_pScriptEngine = pScriptEngine;
	m_DevicePath = DevicePath;
	m_DeviceName = "XOSD";
}

GizmoPluginXOSD::~GizmoPluginXOSD() {
	closePlugin();
	Debug::outDebug("GizmoPluginXOSD :: XOSD Input Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginXOSD::closePlugin() {
	Debug::outDebug("GizmoPluginXOSD :: XOSD Input Plugin Closing...");
	close();
}

bool GizmoPluginXOSD::initPlugin() {
	Debug::outDebug("    XOSD Input Plugin v%s [%s]", PACKAGE_VERSION, m_DevicePath.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginXOSD :: Invalid Script Engine!");
		return false;
	}

	// try to open the device
	if (!init(DEFAULT_FONT, DEFAULT_FONT_SIZE, DEFAULT_FONT_COLOR)) {
		Debug::outWarn("OutputDeviceXOSD :: Failed to Initialize XOSD Output Plugin!");
		return false;
	}
	printString(0, "Gizmo Daemon -=- v%s", PACKAGE_VERSION);
	
	// register functions
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDSetFontName);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDSetFontSize);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDSetFontColor);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDDrawSlider);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDDrawPercentage);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXOSDPrintString);
		
	Debug::out("\t- XOSD Initialized", m_DeviceName.getString(), m_DevicePath.getString());	
	Debug::outDebug("GizmoPluginXOSD :: XOSD Input Plugin Initialized.");
	return true;
}

String GizmoPluginXOSD::getDeviceName() {
	return m_DeviceName;
}

String GizmoPluginXOSD::getDeviceNameAlt() {
	return m_DevicePath;
}

void GizmoPluginXOSD::xosdDisplaySlider(int value){
	displaySlider(SLIDER_LINE, value);
}

void GizmoPluginXOSD::xosdDisplayPercentage(int value){
	displayPercentage(SLIDER_LINE, value);
}

bool GizmoPluginXOSD::xosdGetIsOnScreen(){
	return getXOSDIsOnScreen();
}

void GizmoPluginXOSD::xosdPrintString(int Line, const char * TextToOutput, ...){
	char		text[TEXT_BUFSIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);
			
	printString(Line, text);
}

void GizmoPluginXOSD::xosdSetEnable(bool Enabled){
	setXOSDEnable(Enabled);
}

bool GizmoPluginXOSD::xosdSetFontName(String Font){
	return setXOSDFont(Font);
}

bool GizmoPluginXOSD::xosdSetFontColor(String FontColor){
	return setXOSDFontColor(FontColor);
}

bool GizmoPluginXOSD::xosdSetFontSize(int FontSize) {
	return setXOSDFontSize(FontSize);
}

bool GizmoPluginXOSD::checkEventsWaiting() {
	return false;
}
