  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginXOSD.hpp
//
//	GizmoPluginXOSD header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginXOSD_h
#define __GizmoPluginXOSD_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/GizmoPlugin.hpp"
#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/Debug.hpp"
#include "XOSDInterface.hpp"
#include "XOSD.hpp"
#include "ScriptFunctionXOSDSetFontName.hpp"
#include "ScriptFunctionXOSDSetFontSize.hpp"
#include "ScriptFunctionXOSDSetFontColor.hpp"
#include "ScriptFunctionXOSDDrawPercentage.hpp"
#include "ScriptFunctionXOSDDrawSlider.hpp"
#include "ScriptFunctionXOSDPrintString.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginXOSD : public XOSD, public XOSDInterface {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual bool	onPluginOpened();
	void 		xosdDisplaySlider(int value);
	void		xosdDisplayPercentage(int value);
	bool		xosdGetIsOnScreen();
	void 		xosdPrintString(int Line, const char * TextToOutput, ...);
	void		xosdSetEnable(bool Enabled);
	bool 		xosdSetFontName(String Font);
	bool 		xosdSetFontColor(String FontColor);
	bool 		xosdSetFontSize(int FontSize);

	GizmoPluginXOSD(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginXOSD();
	
private:
	ScriptEngineInterface * m_pScriptEngine;
	
	ScriptFunctionXOSDSetFontName	m_ScriptFuncXOSDSetFontName;
	ScriptFunctionXOSDSetFontColor	m_ScriptFuncXOSDSetFontColor;
	ScriptFunctionXOSDSetFontSize	m_ScriptFuncXOSDSetFontSize;
	ScriptFunctionXOSDDrawSlider	m_ScriptFuncXOSDDrawSlider;
	ScriptFunctionXOSDDrawPercentage m_ScriptFuncXOSDDrawPercentage;
	ScriptFunctionXOSDPrintString	m_ScriptFuncXOSDPrintString;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginXOSD :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginXOSD :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginXOSD(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginXOSD_h
