  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// XOSD.hpp
//
//	XOSD header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "XOSD.hpp"
#include "libGizmo/Debug.hpp"
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

////////////////////////////////////////////////////////////////////////////
// Overridables
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////

const int TEXT_BUFSIZE = 1024;

////////////////////////////////////////////////////////////////////////////
// Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

XOSD::XOSD(int MaxLines) {
	m_xosd = NULL;
	m_MaxLines = MaxLines;
	m_Enabled = true;
}

XOSD::~XOSD() {
	close();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

bool XOSD::init(String Font, int FontSize, String FontColor) {
	m_Font = Font;
	m_FontSize = FontSize;
	m_FontColor = FontColor;
	
	if ((m_xosd = xosd_create(m_MaxLines)) == NULL)
		return false;
		
	Debug::outDebug("XOSD :: Created with [%d] lines, Size [%d], Color [%s]", m_MaxLines, m_FontSize, m_FontColor.getString());

	setXOSDFont(m_Font);
	setXOSDFontColor(m_FontColor);
	xosd_set_timeout(m_xosd, 3);
	xosd_set_pos(m_xosd, XOSD_bottom);
	xosd_set_align(m_xosd, XOSD_center);
	xosd_set_outline_offset(m_xosd, 1);
	xosd_set_shadow_offset(m_xosd, 3);

	return true;
}

void XOSD::close() {
	if (m_xosd) {
		xosd_wait_until_no_display(m_xosd);
		xosd_destroy(m_xosd);
		m_xosd = NULL;
	}
}

void XOSD::printString(int Line, const char * TextToOutput, ...) {
	if ((!m_xosd) || (Line > m_MaxLines - 1) || (!m_Enabled))
		return;

	char		text[TEXT_BUFSIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);
	
	Debug::outDebug("XOSD :: printString(%d) [%s]", m_MaxLines, text);
		
	xosd_display(m_xosd, Line, XOSD_string, text);
}

void XOSD::displaySlider(int Line, int value) {
	if ((!m_xosd) || (Line > m_MaxLines - 1) || (!m_Enabled))
		return;
		
	if (value < 0)
		value = 0;
	else if (value > 100)
		value = 100;

	xosd_display(m_xosd, Line, XOSD_slider, value);
}

void XOSD::displayPercentage(int Line, int value) {
	if ((!m_xosd) || (Line > m_MaxLines - 1) || (!m_Enabled))
		return;
		
	if (value < 0)
		value = 0;
	else if (value > 100)
		value = 100;

	xosd_display(m_xosd, Line, XOSD_percentage, value);
}

void XOSD::setXOSDEnable(bool Enabled) {
	m_Enabled = Enabled;
}

bool XOSD::setXOSDFont(String Font) {
	const int tBUFSIZE = 128;
	char buffer[tBUFSIZE];

	m_Font = Font;

	if (!m_xosd)
		return false;

	snprintf(buffer, sizeof(char) * tBUFSIZE, "%d", m_FontSize);
	String FontString = "-*-";
	FontString += m_Font;
	FontString += "-*-r-*-*-";
	FontString += buffer;
	FontString += "-*-*-*-*-*-*-*";
	if (xosd_set_font(m_xosd, FontString.getString()) == -1) {
		Debug::outError("XOSD :: Failed to Load Font [%s]", FontString.getString());
		return false;
	}

	return true;
}

bool XOSD::setXOSDFontColor(String FontColor) {
	m_FontColor = FontColor;

	if (!m_xosd)
		return false;
		
	xosd_set_colour(m_xosd, m_FontColor.getString());

	return true;
}

bool XOSD::setXOSDFontSize(int FontSize) {
	m_FontSize = FontSize;

	if (!m_xosd)
		return false;
	
	return setXOSDFont(m_Font);
}

bool XOSD::getXOSDIsOnScreen() {
	if (!m_xosd)
		return false;
		
	return xosd_is_onscreen(m_xosd) != 0;
}

